/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is openCRX.
 *
 * The Initial Developer of the Original Code is
 * Josh Geenen <openrx@pirules.net>.
 * Portions created by the Initial Developer are Copyright (C) 2008
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

/* Supported attributes for import:
 * 
 * sourceVcard =
                "BEGIN:VCARD\n" +
                "VERSION:2.1\n" +
                "UID:" + uid.toString() + "\n" +
                "REV:" + rev.substring(0, 15) + "Z\n" +
                "N:\n" +
                "FN:\n" +
                "ORG:\n" +
                "TITLE:\n" +
                "TEL;WORK;VOICE:\n" +
                "TEL;HOME;VOICE:\n" +
                "TEL;CELL;VOICE:\n" +
                (isContact ? "BDAY:\n" : "") +
                "TEL;FAX:\n" +
                "TEL;HOME;FAX:\n" +
                "ADR;WORK;ENCODING=QUOTED-PRINTABLE:;;\n" +
                "ADR;HOME;ENCODING=QUOTED-PRINTABLE:;;\n" +
                "URL;HOME:\n" +
                "URL;WORK:\n" +
                "EMAIL;PREF;INTERNET:\n" +
                "EMAIL;INTERNET:\n" +
                "URL:\n" +
                "END:VCARD";

 * 
 */
var oContactConverter = {
  mV2CMap: {
    UID: "openCrxId",
    FN: "DisplayName",
    NICKNAME: "NickName",
    TITLE: "JobTitle",
    ORG: "Company",
    ADRxPOSTALxWORK: "WorkAddress",
    ADRxPOSTALxHOME: "HomeAddress",
    ADRxWORK: "WorkAddress",
    ADRxHOME: "HomeAddress",
    TELxFAX: "FaxNumber",
    TELxHOMExVOICE: "HomePhone",
    TELxWORKxVOICE: "WorkPhone",
    TELxCELLxVOICE: "CellularNumber",
    TELxPAGER: "PagerNumber",
    URL: "openCrxUrl",
    URLxHOME: "WebPage2",
    URLxWORK: "WebPage1",
    EMAILxPREFxINTERNET: "PrimaryEmail",
    EMAILxINTERNET: "SecondEmail"
  },
  mC2VMap: {
    openCrxId: "UID",
    DisplayName: "FN",
    NickName: "NICKNAME",
    JobTitle: "TITLE",
    Company: "ORG",
  //WorkAddress: "ADR;WORK;ENCODING=QUOTED-PRINTABLE:;;",
  //HomeAddress: "ADR;HOME;ENCODING=QUOTED-PRINTABLE:;;",
    FaxNumber: "TEL;FAX",
    HomePhone: "TEL;HOME;VOICE",
    WorkPhone: "TEL;WORK;VOICE",
    CellularNumber: "TEL;CELL;VOICE",
    PagerNumber: "TEL;PAGER",
    openCrxUrl: "URL",
    WebPage1: "URL;WORK",
    WebPage2: "URL;HOME",
    PrimaryEmail: "EMAIL;PREF;INTERNET",
    SecondEmail: "EMAIL;INTERNET"
  },
  // converts the vCard to a contact AND adds it to the address book
  vCardToContact: function oContactConverter_vCardToContact(aVCard, aAddrBook) {
    if (!aVCard)
      throw "Invalid (null) vCard";
    var contact = aAddrBook.addCard(oAbManager.makeCard());
    var lines = aVCard.split("\n");
    //if (lines[0].toLowerCase() != "begin:vcard");
    //  throw "Invalid vCard: " + aVCard;
    //if (lines[lines.length - 1].toLowerCase() != "end:vcard");
    //  throw "Invalid vCard: " + aVCard;
    for (var i = 1; i < lines.length - 1; i++) {
      try {
        // get this line and split it into the label and value
        var str = lines[i];
        var index   = str.indexOf(":");
        var label   = str.substr(0, index);
        var value   = str.substr(index + 1);
        var origVal = value;
        // strip out all carriage returns
        //value = value.replace(/=0D/g, "");
        // replace the newline with a JS newline
        //value = value.replace(/=0A/g, "\n");
        // special cases: BDAY (birthday), REV (contact's modification date/time),
        //                N (first/last/middle name, etc.),
        //                address with "ENCODING=QUOTED-PRINTABLE"
        if (label == "BDAY") {
          property = "birthday"; // just here in case there is a problem
          var birthday = new Date(this.zTimeToLMD(value));
          oAbManager.setCardValue(contact, "BirthYear", birthday.getUTCFullYear());
          oAbManager.setCardValue(contact, "BirthMonth", birthday.getUTCMonth() + 1);
          oAbManager.setCardValue(contact, "BirthDay", birthday.getUTCDate());
          continue;
        }
        else if (label == "REV") {
          value = this.zTimeToLMD(value);
          property = "LastModifiedDate";
        }
        else if (label == "N") { // names (first;last;etc)
          var names = value.split(";");
          if (names[1]) {
            oLOGGER.LOG("setting FirstName to " + names[1]);
            oAbManager.setCardValue(contact, "FirstName", names[1]);
          }
          if (names[0]) {
            oLOGGER.LOG("setting LastName to " + names[0]);
            oAbManager.setCardValue(contact, "LastName", names[0]);
          }
          continue;
        }
        else if (label.indexOf("ADR") != -1) {
          var vals = value.split(";");
          // get the first two lines of the addresses
          // in Thunderbird, the first address line will have the first 
          var line1 = vals[0];
          var line2 = vals[1];
          var line3 = vals[2];
          if (!line2) {
            line2 = line3;
            line3 = null;
          }
          if (!line1) {
            line1 = line2;
            line2 = line3;
            line3 = null;
          }
          //
          // get the city, state, postal code, and country
          var city = vals[3];
          var state = vals[4];
          var zip = vals[5];
          var country = vals[6];
          // figure out if this is the home or work address
          var type = label.indexOf("HOME") == -1 ? "Work" : "Home";
          oLOGGER.VERBOSE_LOG("Found " + type + " address:\n" + value);
          oLOGGER.VERBOSE_LOG(" - Line 1: " + line1 + "\n - Line 2: " + line2 +
                              "\n - City: " + city + "\n - State: " + state +
                              "\n - Postal Code: " + zip + "\n - Country: " +
                              country);
          // set the values for the card
          oAbManager.setCardValue(contact, type + "Address", line1);
          oAbManager.setCardValue(contact, type + "Address2", line2);
          oAbManager.setCardValue(contact, type + "City", city);
          oAbManager.setCardValue(contact, type + "State", state);
          oAbManager.setCardValue(contact, type + "ZipCode", zip);
          oAbManager.setCardValue(contact, type + "Country", country);
          continue;
        }
        // if it isn't one of those special cases, replace every ';' in the label
        // with an x (just for simplicity) then find the corresponding Thunderbird
        // attribute
        else {
          label = label.replace(/;/g, "x");
          property = this.mV2CMap[label];
        }
        // if there was a Thunderbird property name and value, set those in the
        // contact
        if (property && value) {
          var val = oAbManager.getCardValue(contact, property);
          // if this is an e-mail address & the first is already taken
          if (property == "PrimaryEmail" && val != null && val != "") {
            property = "SecondEmail";
            var val2 = oAbManager.getCardValue(contact, property);
            // if there is already a 2nd e-mail don't reset it
            if (val2 != null && val2 != "") continue;
          }
          oLOGGER.LOG("setting " + property + " to " + value);
          oAbManager.setCardValue(contact, property, value);
        }
      }
      catch (e) {
        LOGGER.LOG_ERROR("Error while setting card value.  Property:" + property + " Value: " + value, e);
      }
    }
    aAddrBook.updateCard(contact);
    return contact;
  },
  contactToVCard: function oContactConverter_contactToVCard(aContact) {
    if (oAbManager.mVersion == 3) {
      var arr = oAbManager.getContactProperties(aContact);
      var propCtr = 0;
      // start off the vCard with BEGIN:VCARD and the version
      var str = "BEGIN:VCARD\nVERSION:2.1\n";
      // add the regular properties
      for (var i = 0; i < arr.length; i++) {
        var property = arr[i].name;
        var value = arr[i].value;
        oLOGGER.VERBOSE_LOG("Found property: " + property + " with value: " + value);
        var vCardProp = this.mC2VMap[property];
        if (vCardProp) {
          propCtr++;
          str += vCardProp + ":" + value + "\n";
        }
      }
      // add the addresses
      var labels = ["Home", "Work"];
      for (var i in labels) {
        var type = labels[i];
        var line1 = oAbManager.getCardValue(aContact, type + "Address");
        var line2 = oAbManager.getCardValue(aContact, type + "Address2");
        var city = oAbManager.getCardValue(aContact, type + "City");
        var state = oAbManager.getCardValue(aContact, type + "State");
        var zip = oAbManager.getCardValue(aContact, type + "ZipCode");
        var country = oAbManager.getCardValue(aContact, type + "Country");
        if (!line2) {
          line2 = line1;
          line1 = null;
        }
        if (line1 || line2 || city || state || zip || country) {
          if (!line1) line1 = "";
          if (!line2) line2 = "";
          if (!city) city = "";
          if (!state) state = "";
          if (!zip) zip = "";
          if (!country) country = "";
          oLOGGER.VERBOSE_LOG("Setting " + type + " address to:\n - Line 1: " +
                              line1 + "\n - Line 2: " + line2 + "\n - City: " +
                              city + "\n - State: " + state + "\n - Postal " +
                              "Code: " + zip + "\n - Country: " + country);
          str += "ADR;" + type.toUpperCase() + ";ENCODING=QUOTED-PRINTABLE:;" +
                  line1 + ";" + line2 + ";" + city + ";" + state + ";" + zip +
                  ";" + country + "\n";
        }
      }

      // add the name
      var name = "";
      var lastName = oAbManager.getCardValue(aContact, "LastName");
      if (lastName)
        name += lastName;
      name += ";";
      var firstName = oAbManager.getCardValue(aContact, "FirstName");
      if (firstName)
        name += firstName;
      name += ";;;";
      if (name != ";;;;;")
        str += "N:" + name + "\n";
      // last modified date
      var lmd = oAbManager.getCardValue(aContact, "LastModifiedDate");
      if (lmd) {
        str += "REV:" + this.LMDToZTime(lmd) + "\n";
      }
      // birthday
      var birthYear = oAbManager.getCardValue(aContact, "BirthYear");
      var birthMonth = oAbManager.getCardValue(aContact, "BirthMonth");
      var birthDate = oAbManager.getCardValue(aContact, "BirthDay");
      if (birthYear && birthMonth && birthDate) {
        birthMonth = parseInt(birthMonth, 10) - 1;
        var birthday = new Date(birthYear, birthMonth, birthDate);
        str += "BDAY: " + this.LMDToZTime(birthday.getTime()/1000) + "\n";
      }
      // finish the card
      str += "END:VCARD\n";
      return propCtr > 0 ? str : "";
    }
    else {
      return "";
    }
  },
  zTimeToLMD: function oContactConverter_zTimeToLMD(sModified) {
    if (!sModified || !sModified.substring) {
      oLOGGER.LOG_WARNING("Invalid string sent to oContactConverter.zTimeToLMD");
      return 0;
    }
    var year  = sModified.substring(0,4);
    var month = sModified.substring(4,6);
    var day   = sModified.substring(6,8);
    var hrs   = sModified.substring(9,11);
    var mins  = sModified.substring(11,13);
    var sec   = sModified.substring(13,15);
    return parseInt(Date.UTC(year, parseInt(month, 10) - 1, day, hrs, mins, sec, 0))/1000;
  },
  //20081026T105219Z 
  LMDToZTime: function oContactConverter_LMDToZTime(sLMD) {
    if (!sLMD) {
      oLOGGER.LOG_WARNING("Invalid sLMD send to oContactConverter.LMDToZTime");
      return "00000000T000000Z";
    }
    var date  = new Date(sLMD * 1000);
    var year  = date.getUTCFullYear();
    var month = new String(parseInt(date.getUTCMonth()) + 1);
    var day   = new String(date.getUTCDate());
    var hrs   = new String(date.getUTCHours());
    var mins  = new String(date.getUTCMinutes());
    var secs  = new String(date.getUTCSeconds());
    for (;month.length < 2; month = "0" + month) {}
    for (;day.length < 2; day = "0" + day) {}
    for (;hrs.length < 2; hrs = "0" + hrs) {}
    for (;mins.length < 2; mins = "0" + mins) {}
    for (;secs.length < 2; secs = "0" + secs) {}
    var str   = "" + year + month + day + "T" + hrs + mins + secs + "Z";
    return str;
  }
};
